import asyncio
from pull_request.pullrequest import PullRequest, PullRequestData
from pull_request.exceptions import InvalidFormatForCredentialFile
from pull_request.command_line_argument_parser import CommandLineArgumentParser
from pull_request.credential_manager import CredentialManager


async def main():
    try:
        argument_parser = CommandLineArgumentParser()
        args = argument_parser.parse_args()

        pr_content = PullRequestData(
            args["from"], args["to"], args["title"], args["body"]
        )

        credential_manager = CredentialManager()
        credentials = credential_manager.read_credential_file()

        pr = PullRequest(credentials, pr_content)
        await pr.make_pull_request()

    except InvalidFormatForCredentialFile:
        print(
            "Error: Make sure your 'pr.json' has all required files: 'username', 'repository' and 'token'."
        )


def _main():
    asyncio.run(main())


if __name__ == "__main__":
    _main()



from pathlib import Path
import json
from .exceptions import CredentialFileDoesNotExist, InvalidFormatForCredentialFile
from dataclasses import dataclass


@dataclass
class Credentials:
    username: str
    repository_name: str
    token: str

    def get_token(self) -> str:
        return "Bearer " + self.token


class CredentialManager:
    REQUIRED_CREDENTIAL_FILE_KEYS = {"username", "repository", "token"}

    def __init__(self) -> None:
        self._pr_json_path = Path("pr.json")

    def pr_json_file_exists(self) -> bool:
        return self._pr_json_path.is_file()

    def has_all_required_keys(self, credential_file: dict) -> bool:
        credential_file_keys = set(credential_file.keys())
        return all(
            key in credential_file_keys for key in self.REQUIRED_CREDENTIAL_FILE_KEYS
        )

    def read_credential_file(self) -> Credentials:
        if not self.pr_json_file_exists():
            raise CredentialFileDoesNotExist()

        with open(self._pr_json_path) as file:
            pr_json_file = json.loads(file.read())

            if not self.has_all_required_keys(pr_json_file):
                raise InvalidFormatForCredentialFile()

            return Credentials(
                pr_json_file["username"],
                pr_json_file["repository"],
                pr_json_file["token"],
            )



from argparse import ArgumentParser


class CommandLineArgumentParser:
    def __init__(self) -> None:
        self._parser = ArgumentParser(
            description="Pull requests on command line - Author: Icarus"
        )
        self.add_commands()

    @property
    def parser(self) -> ArgumentParser:
        return self._parser

    def add_commands(self) -> None:
        self.parser.add_argument(
            "-f", "--from", help="Changes are coming from this branch", required=True
        )
        self.parser.add_argument(
            "-t", "--to", help="Changes are going to that branch", required=True
        )
        self.parser.add_argument(
            "-T", "--title", help="Pull request title", required=True
        )
        self.parser.add_argument("-b", "--body", help="Body of the pull request")

    def parse_args(self) -> dict:
        args = self.parser.parse_args()
        return vars(args)



from setuptools import setup

long_description = None
with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="pull-requests",
    version="1.1.1",
    description="A command line interface for creating pull requests",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Hícaro Dânrlley",
    author_email="hdanrlley1@gmail.com",
    packages=["pull_request"],
    install_requires=["requests"],  # external packages as dependencies
    python_requires=">=3.10",
    entry_points={
        "console_scripts": [
            "pull_request = pull_request.main:_main",
        ]
    },
)
